@@ -46,6 +46,7 @@
 #include <vector>
 
 #include "WPXOLEStream.h"
+#include "WPXStream.h"
 #include <string.h>
 
 namespace libwpd
@@ -68,9 +69,7 @@ public:
 
 	Header();
 	bool valid();
-	void load( const unsigned char *buffer );
-	void save( unsigned char *buffer );
-	void debug();
+	void load( const unsigned char *buffer, unsigned long size );
 };
 
 class AllocTable
@@ -85,14 +84,10 @@ public:
 	void clear();
 	unsigned long count();
 	void resize( unsigned long newsize );
-	void preserve( unsigned long n );
 	void set( unsigned long index, unsigned long val );
-	unsigned unused();
-	void setChain( std::vector<unsigned long> );
 	std::vector<unsigned long> follow( unsigned long start );
 	unsigned long operator[](unsigned long index );
 	void load( const unsigned char *buffer, unsigned len );
-	void save( unsigned char *buffer );
 private:
 	std::vector<unsigned long> data;
 	AllocTable( const AllocTable & );
@@ -103,7 +98,7 @@ class DirEntry
 {
 public:
 	DirEntry() : valid(false), name(), dir(false), size(0), start(0),
-		prev(0), next(0), child(0) {};
+		prev(0), next(0), child(0) {}
 	bool valid;            // false if invalid (should be skipped)
 	std::string name;      // the name, not in unicode anymore
 	bool dir;              // true if directory
@@ -123,12 +118,8 @@ public:
 	unsigned entryCount();
 	DirEntry *entry( unsigned index );
 	DirEntry *entry( const std::string &name );
-	int parent( unsigned index );
-	std::string fullName( unsigned index );
-	std::vector<unsigned> children( unsigned index );
 	unsigned find_child( unsigned index, const std::string &name );
 	void load( unsigned char *buffer, unsigned len );
-	void save( unsigned char *buffer );
 private:
 	std::vector<DirEntry> entries;
 	DirTree( const DirTree & );
@@ -139,9 +130,8 @@ class StorageIO
 {
 public:
 	Storage *storage;         // owner
-	std::stringstream buf;
+	WPXInputStream *input;
 	int result;               // result of operation
-	unsigned long bufsize;    // size of the buffer
 
 	Header *header;           // storage header
 	DirTree *dirtree;         // directory tree
@@ -150,9 +140,7 @@ public:
 
 	std::vector<unsigned long> sb_blocks; // blocks for "small" files
 
-	std::list<Stream *> streams;
-
-	StorageIO( Storage *storage, const std::stringstream &memorystream );
+	StorageIO( Storage *storage, WPXInputStream *is );
 	~StorageIO();
 
 	bool isOLEStream();
@@ -188,7 +176,6 @@ public:
 	~StreamIO();
 	unsigned long size();
 	unsigned long tell();
-	int getch();
 	unsigned long read( unsigned char *data, unsigned long maxlen );
 	unsigned long read( unsigned long pos, unsigned char *data, unsigned long maxlen );
 
@@ -204,7 +191,7 @@ private:
 	unsigned long m_pos;
 
 	// simple cache system to speed-up getch()
-	unsigned char *cache_data;
+	std::vector<unsigned char> cache_data;
 	unsigned long cache_size;
 	unsigned long cache_pos;
 	void updateCache();
@@ -214,12 +201,12 @@ private:
 
 static inline unsigned long readU16( const unsigned char *ptr )
 {
-	return ptr[0]+(ptr[1]<<8);
+	return (unsigned long)(unsigned)(ptr[0]+(ptr[1]<<8));
 }
 
 static inline unsigned long readU32( const unsigned char *ptr )
 {
-	return ptr[0]+(ptr[1]<<8)+(ptr[2]<<16)+(ptr[3]<<24);
+	return (unsigned long)(unsigned)(ptr[0]+(ptr[1]<<8)+(ptr[2]<<16)+(ptr[3]<<24));
 }
 
 static const unsigned char wpsole_magic[] =
@@ -258,8 +245,10 @@ bool libwpd::Header::valid()
 	return true;
 }
 
-void libwpd::Header::load( const unsigned char *buffer )
+void libwpd::Header::load( const unsigned char *buffer, unsigned long size )
 {
+	if (size < 512)
+		return;
 	b_shift      = readU16( buffer + 0x1e );
 	s_shift      = readU16( buffer + 0x20 );
 	num_bat      = readU32( buffer + 0x2c );
@@ -307,14 +296,6 @@ void libwpd::AllocTable::resize( unsigned long newsize )
 			data[i] = Avail;
 }
 
-// make sure there're still free blocks
-void libwpd::AllocTable::preserve( unsigned long n )
-{
-	std::vector<unsigned long> pre;
-	for( unsigned i=0; i < n; i++ )
-		pre.push_back( unused() );
-}
-
 unsigned long libwpd::AllocTable::operator[]( unsigned long index )
 {
 	unsigned long result;
@@ -328,16 +309,6 @@ void libwpd::AllocTable::set( unsigned long index, unsigned long value )
 	data[ index ] = value;
 }
 
-void libwpd::AllocTable::setChain( std::vector<unsigned long> chain )
-{
-	if( chain.size() )
-	{
-		for( unsigned i=0; i<chain.size()-1; i++ )
-			set( chain[i], chain[i+1] );
-		set( chain[ chain.size()-1 ], AllocTable::Eof );
-	}
-}
-
 // TODO: optimize this with better search
 static bool already_exist(const std::vector<unsigned long>& chain,
 unsigned long item)
@@ -370,19 +341,6 @@ std::vector<unsigned long> libwpd::AllocTable::follow( unsigned long start )
 	return chain;
 }
 
-unsigned libwpd::AllocTable::unused()
-{
-	// find first available block
-	for( unsigned i = 0; i < data.size(); i++ )
-		if( data[i] == Avail )
-			return i;
-
-	// completely full, so enlarge the table
-	unsigned block = data.size();
-	resize( data.size()+10 );
-	return block;
-}
-
 void libwpd::AllocTable::load( const unsigned char *buffer, unsigned len )
 {
 	resize( len / 4 );
@@ -425,62 +383,6 @@ libwpd::DirEntry *libwpd::DirTree::entry( unsigned index )
 	return &entries[ index ];
 }
 
-int libwpd::DirTree::parent( unsigned index )
-{
-	// brute-force, basically we iterate for each entries, find its children
-	// and check if one of the children is 'index'
-	for( unsigned j=0; j<entryCount(); j++ )
-	{
-		std::vector<unsigned> chi = children( j );
-		for( unsigned i=0; i<chi.size(); i++ )
-			if( chi[i] == index )
-				return j;
-	}
-
-	return -1;
-}
-
-std::string libwpd::DirTree::fullName( unsigned index )
-{
-	// don't use root name ("Root Entry"), just give "/"
-	if( index == 0 ) return "/";
-
-	std::string result = entry( index )->name;
-	result.insert( 0,  "/" );
-	int p = parent( index );
-	DirEntry *_entry = 0;
-
-	std::vector<int> seens;
-	seens.push_back(p);
-
-	while( p > 0 )
-	{
-		_entry = entry( p );
-		if (_entry->dir && _entry->valid)
-		{
-			result.insert( 0,  _entry->name);
-			result.insert( 0,  "/" );
-		}
-
-		p = parent(p);
-		if (p < 0) break;
-
-		bool ok = true;
-		// sanity check
-		for (int i = 0; i < int(seens.size()); i++)
-		{
-			if (seens[i] == p)
-			{
-				ok = false;
-				break;
-			}
-		}
-		if (!ok) break;
-		seens.push_back(p);
-	}
-	return result;
-}
-
 // given a fullname (e.g "/ObjectPool/_1020961869"), find the entry
 libwpd::DirEntry *libwpd::DirTree::entry( const std::string &name )
 {
@@ -522,40 +424,6 @@ libwpd::DirEntry *libwpd::DirTree::entry( const std::string &name )
 	return entry( index );
 }
 
-// helper function: recursively find siblings of index
-static void dirtree_find_siblings( libwpd::DirTree *dirtree, std::vector<unsigned>& result,
-unsigned index )
-{
-	libwpd::DirEntry *e = dirtree->entry( index );
-	if( !e ) return;
-	if( !e->valid ) return;
-
-	// prevent infinite loop
-	for( unsigned i = 0; i < result.size(); i++ )
-		if( result[i] == index ) return;
-
-	// add myself
-	result.push_back( index );
-
-	// visit previous sibling, don't go infinitely
-	unsigned prev = e->prev;
-	if( ( prev > 0 ) && ( prev < dirtree->entryCount() ) )
-	{
-		for( unsigned i = 0; i < result.size(); i++ )
-			if( result[i] == prev ) prev = 0;
-		if( prev ) dirtree_find_siblings( dirtree, result, prev );
-	}
-
-	// visit next sibling, don't go infinitely
-	unsigned next = e->next;
-	if( ( next > 0 ) && ( next < dirtree->entryCount() ) )
-	{
-		for( unsigned i = 0; i < result.size(); i++ )
-			if( result[i] == next ) next = 0;
-		if( next ) dirtree_find_siblings( dirtree, result, next );
-	}
-}
-
 static unsigned dirtree_find_sibling( libwpd::DirTree *dirtree, unsigned index, const std::string &name )
 {
 
@@ -590,17 +458,6 @@ unsigned libwpd::DirTree::find_child( unsigned index, const std::string &name )
 	return 0;
 }
 
-std::vector<unsigned> libwpd::DirTree::children( unsigned index )
-{
-	std::vector<unsigned> result;
-
-	DirEntry *e = entry( index );
-	if( e ) if( e->valid && e->child < entryCount() )
-			dirtree_find_siblings( this, result, e->child );
-
-	return result;
-}
-
 void libwpd::DirTree::load( unsigned char *buffer, unsigned size )
 {
 	entries.clear();
@@ -644,17 +501,15 @@ void libwpd::DirTree::load( unsigned char *buffer, unsigned size )
 
 // =========== StorageIO ==========
 
-libwpd::StorageIO::StorageIO( libwpd::Storage *st, const std::stringstream &memorystream ) :
+libwpd::StorageIO::StorageIO( libwpd::Storage *st, WPXInputStream *is ) :
 	storage(st),
-	buf( memorystream.str(), std::ios::binary | std::ios::in ),
+	input( is ),
 	result(libwpd::Storage::Ok),
-	bufsize(0),
 	header(new libwpd::Header()),
 	dirtree(new libwpd::DirTree()),
 	bbat(new libwpd::AllocTable()),
 	sbat(new libwpd::AllocTable()),
-	sb_blocks(),
-	streams()
+	sb_blocks()
 {
 	bbat->blockSize = 1 << header->b_shift;
 	sbat->blockSize = 1 << header->s_shift;
@@ -666,10 +521,6 @@ libwpd::StorageIO::~StorageIO()
 	delete bbat;
 	delete dirtree;
 	delete header;
-
-	std::list<libwpd::Stream *>::iterator it;
-	for( it = streams.begin(); it != streams.end(); ++it )
-		delete *it;
 }
 
 bool libwpd::StorageIO::isOLEStream()
@@ -680,23 +531,19 @@ bool libwpd::StorageIO::isOLEStream()
 
 void libwpd::StorageIO::load()
 {
-	unsigned char *buffer = 0;
-	unsigned long buflen = 0;
 	std::vector<unsigned long> blocks;
 
-	// find size of input file
-	buf.seekg( 0, std::ios::end );
-	bufsize = buf.tellg();
-
 	// load header
-	buffer = new unsigned char[512];
-	buf.seekg( 0 );
-	buf.read( (char *)buffer, 512 );
-	header->load( buffer );
-	delete[] buffer;
+	unsigned long numBytesRead = 0;
+	const unsigned char *buf = input->read(512, numBytesRead);
+
+	result = libwpd::Storage::NotOLE;
+	if (numBytesRead < 512)
+		return;
+
+	header->load( buf, numBytesRead );
 
 	// check OLE magic id
-	result = libwpd::Storage::NotOLE;
 	for( unsigned i=0; i<8; i++ )
 		if( header->id[i] != wpsole_magic[i] )
 			return;
@@ -719,7 +566,7 @@ void libwpd::StorageIO::load()
 		else blocks[j] = header->bb_blocks[j];
 	if( (header->num_bat > 109) && (header->num_mbat > 0) )
 	{
-		unsigned char *buffer2 = new unsigned char[ bbat->blockSize ];
+		std::vector<unsigned char> buffer2( bbat->blockSize );
 		unsigned k = 109;
 		unsigned sector;
 		for( unsigned r = 0; r < header->num_mbat; r++ )
@@ -728,55 +575,40 @@ void libwpd::StorageIO::load()
 				sector = header->mbat_start;
 			else      // next meta bat location is the last current block value.
 				sector = blocks[--k];
-			loadBigBlock( sector, buffer2, bbat->blockSize );
+			loadBigBlock( sector, &buffer2[0], bbat->blockSize );
 			for( unsigned s=0; s < bbat->blockSize; s+=4 )
 			{
 				if( k >= header->num_bat ) break;
-				else  blocks[k++] = readU32( buffer2 + s );
+				else  blocks[k++] = readU32( &buffer2[s] );
 			}
-			/*
-			loadBigBlock( header->mbat_start+r, buffer2, bbat->blockSize );
-			for( unsigned s=0; s < bbat->blockSize; s+=4 )
-			{
-			  if( k >= header->num_bat ) break;
-			  else  blocks[k++] = readU32( buffer2 + s );
-			}
-			*/
 		}
-		delete[] buffer2;
 	}
 
 	// load big bat
-	buflen = blocks.size()*bbat->blockSize;
-	if( buflen > 0 )
+	if( blocks.size()*bbat->blockSize > 0 )
 	{
-		buffer = new unsigned char[ buflen ];
-		loadBigBlocks( blocks, buffer, buflen );
-		bbat->load( buffer, buflen );
-		delete[] buffer;
+		std::vector<unsigned char> buffer( blocks.size()*bbat->blockSize );
+		loadBigBlocks( blocks, &buffer[0], buffer.size() );
+		bbat->load( &buffer[0], buffer.size() );
 	}
 
 	// load small bat
 	blocks.clear();
 	blocks = bbat->follow( header->sbat_start );
-	buflen = blocks.size()*bbat->blockSize;
-	if( buflen > 0 )
+	if( blocks.size()*bbat->blockSize > 0 )
 	{
-		buffer = new unsigned char[ buflen ];
-		loadBigBlocks( blocks, buffer, buflen );
-		sbat->load( buffer, buflen );
-		delete[] buffer;
+		std::vector<unsigned char> buffer( blocks.size()*bbat->blockSize );
+		loadBigBlocks( blocks, &buffer[0], buffer.size() );
+		sbat->load( &buffer[0], buffer.size() );
 	}
 
 	// load directory tree
 	blocks.clear();
 	blocks = bbat->follow( header->dirent_start );
-	buflen = blocks.size()*bbat->blockSize;
-	buffer = new unsigned char[ buflen ];
-	loadBigBlocks( blocks, buffer, buflen );
-	dirtree->load( buffer, buflen );
-	unsigned sb_start = readU32( buffer + 0x74 );
-	delete[] buffer;
+	std::vector<unsigned char> buffer(blocks.size()*bbat->blockSize);
+	loadBigBlocks( blocks, &buffer[0], buffer.size() );
+	dirtree->load( &buffer[0], buffer.size() );
+	unsigned sb_start = readU32( &buffer[0x74] );
 
 	// fetch block chain as data for small-files
 	sb_blocks = bbat->follow( sb_start ); // small files
@@ -818,10 +650,12 @@ unsigned char *data, unsigned long maxlen )
 		unsigned long block = blocks[i];
 		unsigned long pos =  bbat->blockSize * ( block+1 );
 		unsigned long p = (bbat->blockSize < maxlen-bytes) ? bbat->blockSize : maxlen-bytes;
-		if( pos + p > bufsize ) p = bufsize - pos;
-		buf.seekg( pos );
-		buf.read( (char *)data + bytes, p );
-		bytes += p;
+
+		input->seek(pos, WPX_SEEK_SET);
+		unsigned long numBytesRead = 0;
+		const unsigned char *buf = input->read(p, numBytesRead);
+		memcpy(data+bytes, buf, numBytesRead);
+		bytes += numBytesRead;
 	}
 
 	return bytes;
@@ -851,7 +685,7 @@ unsigned char *data, unsigned long maxlen )
 	if( maxlen == 0 ) return 0;
 
 	// our own local buffer
-	unsigned char *tmpBuf = new unsigned char[ bbat->blockSize ];
+	std::vector<unsigned char> tmpBuf( bbat->blockSize );
 
 	// read small block one by one
 	unsigned long bytes = 0;
@@ -864,18 +698,16 @@ unsigned char *data, unsigned long maxlen )
 		unsigned long bbindex = pos / bbat->blockSize;
 		if( bbindex >= sb_blocks.size() ) break;
 
-		loadBigBlock( sb_blocks[ bbindex ], tmpBuf, bbat->blockSize );
+		loadBigBlock( sb_blocks[ bbindex ], &tmpBuf[0], bbat->blockSize );
 
 		// copy the data
 		unsigned offset = pos % bbat->blockSize;
 		unsigned long p = (maxlen-bytes < bbat->blockSize-offset ) ? maxlen-bytes :  bbat->blockSize-offset;
 		p = (sbat->blockSize<p ) ? sbat->blockSize : p;
-		memcpy( data + bytes, tmpBuf + offset, p );
+		memcpy( data + bytes, &tmpBuf[offset], p );
 		bytes += p;
 	}
 
-	delete[] tmpBuf;
-
 	return bytes;
 }
 
@@ -903,7 +735,7 @@ libwpd::StreamIO::StreamIO( libwpd::StorageIO *s, libwpd::DirEntry *e) :
 	fail(false),
 	blocks(),
 	m_pos(0),
-	cache_data(0),
+	cache_data(),
 	cache_size(4096),
 	cache_pos(0)
 {
@@ -913,14 +745,13 @@ libwpd::StreamIO::StreamIO( libwpd::StorageIO *s, libwpd::DirEntry *e) :
 		blocks = io->sbat->follow( entry->start );
 
 	// prepare cache
-	cache_data = new unsigned char[cache_size];
+	cache_data = std::vector<unsigned char>(cache_size);
 	updateCache();
 }
 
 // FIXME tell parent we're gone
 libwpd::StreamIO::~StreamIO()
 {
-	delete[] cache_data;
 }
 
 unsigned long libwpd::StreamIO::tell()
@@ -928,25 +759,6 @@ unsigned long libwpd::StreamIO::tell()
 	return m_pos;
 }
 
-int libwpd::StreamIO::getch()
-{
-	// past end-of-file ?
-	if( m_pos > entry->size ) return -1;
-
-	// need to update cache ?
-	if( !cache_size || ( m_pos < cache_pos ) ||
-	( m_pos >= cache_pos + cache_size ) )
-		updateCache();
-
-	// something bad if we don't get good cache
-	if( !cache_size ) return -1;
-
-	int data = cache_data[m_pos - cache_pos];
-	m_pos++;
-
-	return data;
-}
-
 unsigned long libwpd::StreamIO::read( unsigned long pos, unsigned char *data, unsigned long maxlen )
 {
 	// sanity checks
@@ -962,21 +774,19 @@ unsigned long libwpd::StreamIO::read( unsigned long pos, unsigned char *data, un
 
 		if( index >= blocks.size() ) return 0;
 
-		unsigned char *buf = new unsigned char[ io->sbat->blockSize ];
+		std::vector<unsigned char> buf( io->sbat->blockSize );
 		unsigned long offset = pos % io->sbat->blockSize;
 		while( totalbytes < maxlen )
 		{
 			if( index >= blocks.size() ) break;
-			io->loadSmallBlock( blocks[index], buf, io->bbat->blockSize );
+			io->loadSmallBlock( blocks[index], &buf[0], io->bbat->blockSize );
 			unsigned long count = io->sbat->blockSize - offset;
 			if( count > maxlen-totalbytes ) count = maxlen-totalbytes;
-			memcpy( data+totalbytes, buf + offset, count );
+			memcpy( data+totalbytes, &buf[offset], count );
 			totalbytes += count;
 			offset = 0;
 			index++;
 		}
-		delete[] buf;
-
 	}
 	else
 	{
@@ -985,21 +795,19 @@ unsigned long libwpd::StreamIO::read( unsigned long pos, unsigned char *data, un
 
 		if( index >= blocks.size() ) return 0;
 
-		unsigned char *buf = new unsigned char[ io->bbat->blockSize ];
+		std::vector<unsigned char> buf( io->bbat->blockSize );
 		unsigned long offset = pos % io->bbat->blockSize;
 		while( totalbytes < maxlen )
 		{
 			if( index >= blocks.size() ) break;
-			io->loadBigBlock( blocks[index], buf, io->bbat->blockSize );
+			io->loadBigBlock( blocks[index], &buf[0], io->bbat->blockSize );
 			unsigned long count = io->bbat->blockSize - offset;
 			if( count > maxlen-totalbytes ) count = maxlen-totalbytes;
-			memcpy( data+totalbytes, buf + offset, count );
+			memcpy( data+totalbytes, &buf[offset], count );
 			totalbytes += count;
 			index++;
 			offset = 0;
 		}
-		delete [] buf;
-
 	}
 
 	return totalbytes;
@@ -1015,21 +823,21 @@ unsigned long libwpd::StreamIO::read( unsigned char *data, unsigned long maxlen
 void libwpd::StreamIO::updateCache()
 {
 	// sanity check
-	if( !cache_data ) return;
+	if( cache_data.empty() ) return;
 
 	cache_pos = m_pos - ( m_pos % cache_size );
 	unsigned long bytes = cache_size;
 	if( cache_pos + bytes > entry->size ) bytes = entry->size - cache_pos;
-	cache_size = read( cache_pos, cache_data, bytes );
+	cache_size = read( cache_pos, &cache_data[0], bytes );
 }
 
 
 // =========== Storage ==========
 
-libwpd::Storage::Storage( const std::stringstream &memorystream ) :
+libwpd::Storage::Storage( WPXInputStream *is ) :
 	io(NULL)
 {
-	io = new StorageIO( this, memorystream );
+	io = new StorageIO( this, is );
 }
 
 libwpd::Storage::~Storage()
